local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

local idEnchantingVellum = 38682

local partsMemory = { }
local resourcesMemory = { }

function Capitalist:PartMemoryReset()
	partsMemory = { }
end
function Capitalist:ResourceMemoryReset()
	resourcesMemory = { }
end

function Capitalist:IsComponent(componentName)
	if (self.db.profile.components and
		self.db.profile.components[componentName]) then

		return true
	end
	return false
end

function Capitalist:IsPart(componentName)
	return (self:IsComponent(componentName) and
			self:IsProduct(componentName))
end
function Capitalist:IsResource(componentName)
	return (self:IsComponent(componentName) and
			(not self:IsProduct(componentName)))
end

function Capitalist:GetComponentId(componentName)
	if (componentName == L["Enchanting Vellum"]) then
		return idEnchantingVellum
	else
		return self.db.profile.components[componentName].id
	end
end

function Capitalist:ResourceGetMerchant(resourceName)
	return self.db.profile.components[resourceName].merchant
end

function Capitalist:ResourceSetMerchant(resourceName, merchant)
	self.db.profile.components[resourceName].merchant = merchant
end

function Capitalist:OutputComponent(prefix, itemName, count)
	self:Print(self:StringForItem(prefix, self:GetComponentText(itemName), count).." @ "..self:PrettyCurrency(self:GetResourceCopperCost(itemName)).." ea.")
end

function Capitalist:OutputMissingCosts()
	local resources = {}
	for resourceName, resourceData in pairs(resourcesMemory) do
		table.insert(resources, resourceName)
	end

	local outputtedHeaderCost

	-- TODO: skip the go-between resources table (now that resourcesMemory is available, vs. componentsMemory)

	i = 1
	while (i <= #resources) do
		local resourceName = resources[i]

		if (not self.db.profile.components[resourceName].cost) then
			if (not outputtedHeaderCost) then
				self:Print(L["Resources missing cost information:"])
				outputtedHeaderCost = true
			end
			self:Print(string.format("  "..self:GetComponentText(resourceName)))
		end

		i = i + 1
	end

	if (not outputtedHeaderCost) then
		self:Print(L["All resource buy prices and costs set."])
	end
end

function Capitalist:GetComponentText(itemName)
	if (self.db.profile.components and
		self.db.profile.components[itemName] and
		self.db.profile.components[itemName].link) then

		return self.db.profile.components[itemName].link
	end
	
	return itemName
end

function Capitalist:PartIsVisible(partName)
	return partsMemory[partName] ~= nil
end
function Capitalist:PartSetVisible(partName, visible)
	if (visible) then
		partsMemory[partName] = { }
	else
		partsMemory[partName] = nil
	end
end

function Capitalist:ResourceIsVisible(componentName)
	return resourcesMemory[componentName] ~= nil
end
function Capitalist:ResourceSetVisible(componentName, visible)
	if (visible) then
		resourcesMemory[componentName] = { }
	else
		resourcesMemory[componentName] = nil
	end
end

-- REVIEW: Remove if unused after a few builds
function Capitalist:GetVisibleParts()
	local partList = {}
	for partName, partData in pairs(partMemory) do
		table.insert(partList, partName)
	end
	return partList
end

function Capitalist:GetVisibleResources()
	local resourceList = {}
	for resourceName, resourceData in pairs(resourcesMemory) do
		table.insert(resourceList, resourceName)
	end
	return resourceList
end

function Capitalist:GetVisibleComponents()
	local componentList = {}
	for resourceName, resourceData in pairs(resourcesMemory) do
		table.insert(componentList, resourceName)
	end
	for partName, partData in pairs(partsMemory) do
		table.insert(componentList, partName)
	end
	return componentList
end

function Capitalist:GetSortedVisibleComponents()
	local componentList = self:GetVisibleComponents()
	table.sort(componentList, function(a, b) return a < b end)
	return componentList
end

function Capitalist:StringForItem(prefix, item, count)
	local s = item;
	if (prefix) then
		s = prefix..s
	end
	if (count and (count > 1)) then
		s = s.." ["..count.."]"
	end
	return s
end
